from pathlib import Path
import argparse
import shutil

import yaml

FILES = {
    'counts':'counts.txt',
    'metadata': 'samples.txt',
    'taxonomy': 'taxonomy.txt',
    'tree': 'tree.tre'
}

def main():

    parser = argparse.ArgumentParser(
        description=(
            "Retrieve and standardize data files from the sarcoid subprojects"))
    parser.add_argument(
        "--data_files_fp",
        help=(
            "Path to the data_files directory tree, with manifest.yml files "
            "in each directory (default: %(default)s)"),
        metavar="DIR",
        default=".",
        dest="data_fp",
        type=_check_fp)

    args = parser.parse_args()

    root_manifest_fp = args.data_fp/"manifest.yml"
    with root_manifest_fp.open() as root_manifest:
        y = yaml.load(root_manifest)
        root = _check_fp(y['root'])

    manifests = args.data_fp.glob("**/manifest.yml")

    for manifest in manifests:
        if manifest == root_manifest_fp: continue
        try:
            m = _parse_manifest(manifest, root)
        except:
            raise ValueError("Error parsing manifest %s" % manifest)

        
def _parse_manifest(manifest, global_root):
    m = None
    with manifest.open() as _m:
        m = yaml.load(_m)
    dest_fp = manifest.parent.resolve().absolute()
    meta = m['meta']
    paths = m['paths']
    root = _check_fp(global_root/paths['root'])
    print("For sample set %s (%s):" % (meta.get("sampleset"), meta.get("method")))
    for f, fname in FILES.items():
        if f in paths:
            src = root/paths[f]
            if not src.exists():
                raise ValueError("In %s:%s, %s does not exist" % (manifest, f, src))
            dest = dest_fp/fname
            print("\tCopying %s...\n\t\t%s ==>\n\t\t%s" % (f, src, dest))
            shutil.copy(str(src), str(dest))
        else:
            print("\t NOTE: %s not found in %s; skipping" % (f, manifest))
            
def _check_fp(fp):
    p = Path(fp)
    if not p.is_dir():
        raise argparse.ArgumentTypeError("%s is not a directory" % fp)
    return p
    

if __name__ == "__main__":
    main()


